@@ -1,4 +1,4 @@
-from pyrf.config import SweepEntry, TriggerSettings, TriggerSettingsError
+from pyrf.config import SweepEntry, TriggerSettings, TRIGGER_TYPE_LEVEL, TRIGGER_TYPE_NONE, TriggerSettingsError
 from pyrf.connectors.blocking import PlainSocketConnector
 from pyrf.connectors.base import sync_async
 from pyrf.vrt import vrt_packet_reader, I_ONLY, IQ
@@ -27,6 +27,7 @@ class WSA4000Properties(object):
 
     RFE_MODES = ('ZIF',)
 
+    DEFAULT_SAMPLE_TYPE = {'ZIF': IQ} # almost true, see CAPTURE_FREQ_RANGES
     FULL_BW = {'ZIF': 125*M}
     USABLE_BW = {'ZIF': 90*M}
     MIN_TUNABLE = {'ZIF': 90*M}
@@ -42,6 +43,22 @@ class WSA4000Properties(object):
         'antenna', 'gain', 'ifgain', 'spp', 'ppb', 'dwell_s', 'dwell_us',
         'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
 
+    SPECA_DEFAULTS = {
+        'mode': 'ZIF',
+        'center': 2450 * M,
+        'rbw': 122070,
+        'span': 125 * M,
+        'decimation': 1,
+        'fshift': 0,
+        'device_settings': {
+            'antenna': 1,
+            'ifgain': 0,
+            'gain': 'low',
+            },
+        'device_class': 'thinkrf.WSA',
+        'device_identifier': 'unknown',
+        }
+
 
 class WSA5000_220Properties(object):
     model = 'WSA5000-220'
@@ -53,25 +70,110 @@ class WSA5000_220Properties(object):
     SWEEP_FREQ_RANGE = (100*M, 20000*M)
     RFE_ATTENUATION = 20
     RFE_MODES = ('ZIF', 'SH', 'SHN', 'HDR', 'IQIN', 'DD')
-
-    FULL_BW = {'ZIF': 125*M, 'HDR': 162760, 'SH': 62500000 , 'SHN': 62500000, 'IQIN': 62500000, 'DD': 62500000}
-    USABLE_BW = {'ZIF': 100*M, 'HDR': 100000, 'SH': 40000000,'SHN': 40000000, 'IQIN': 50000000, 'DD': 62500000}
-    MIN_TUNABLE = {'ZIF': 50*M, 'HDR': 50*M, 'SH': 50*M, 'SHN': 50*M,'IQIN': 0.1*M, 'DD': 31.2*M}
-    MAX_TUNABLE = {'ZIF': 20000*M, 'HDR': 20000*M, 'SH': 20000*M,'SHN': 20000*M, 'IQIN': 0.1*M, 'DD': 31.2*M}
-    MIN_DECIMATION = {'ZIF': 4, 'HDR': None, 'SH': 4, 'SHN': 4,'IQIN': None, 'DD': None}
-    MAX_DECIMATION = {'ZIF': 1024, 'HDR': None, 'SH': 4, 'SHN': 4,'IQIN': None, 'DD': None}
+    DEFAULT_SAMPLE_TYPE = {
+        'ZIF': IQ,
+        'SH': I_ONLY,
+        'SHN': I_ONLY,
+        'HDR': I_ONLY,
+        'IQIN': IQ,
+        'DD': I_ONLY,
+        }
+    FULL_BW = {
+        'ZIF': 125 * M,
+        'HDR': 0.16276 * M,
+        'SH': 62.5 * M,
+        'SHN': 62.5 * M,
+        'IQIN': 125 * M,
+        'DD': 62.5 * M,
+        }
+    USABLE_BW = {
+        'ZIF': 100 * M,
+        'HDR': 0.1 * M,
+        'SH': 40 * M,
+        'SHN': 10 * M,
+        'IQIN': 100 * M,
+        'DD': 62.5 * M,
+        }
+    MIN_TUNABLE = {
+        'ZIF': 50 * M,
+        'HDR': 50 * M,
+        'SH': 50 * M,
+        'SHN': 50 * M,
+        'IQIN': 0,
+        'DD': 31.24 * M,
+        }
+    MAX_TUNABLE = {
+        'ZIF': 20000 * M,
+        'HDR': 20000 * M,
+        'SH': 20000 * M,
+        'SHN': 20000 * M,
+        'IQIN': 0,
+        'DD': 31.24 * M,
+        }
+    MIN_DECIMATION = {
+        'ZIF': 4,
+        'HDR': None,
+        'SH': 4,
+        'SHN': 4,
+        'IQIN': 4,
+        'DD': 4,
+        }
+    MAX_DECIMATION = {
+        'ZIF': 1024,
+        'HDR': None,
+        'SH': 4,
+        'SHN': 4,
+        'IQIN': 1024,
+        'DD': 1024,
+        }
     DECIMATED_USABLE = 0.80
-    PASS_BAND_CENTER = {'ZIF': 0.5, 'HDR': 0.6, 'SH': 0.56, 'SHN': 0.45,'IQIN': 0.5, 'DD': 0.5}
-    PASS_BAND_OFFSET = {'SHN': 7*M}
+    PASS_BAND_CENTER = {
+        'ZIF': 0.5,
+        'HDR': 0.6,
+        'SH': 0.56,
+        'SHN': 0.56,
+        'IQIN': 0.5,
+        'DD': 0.5,
+        }
     DC_OFFSET_BW = 240000 # XXX: an educated guess
     TUNING_RESOLUTION = 100000
-    FSHIFT_AVAILABLE = {'ZIF': True, 'HDR': False, 'SH': True, 'SHN': True,'IQIN': False, 'DD': False}
+    FSHIFT_AVAILABLE = {
+        'ZIF': True,
+        'HDR': False,
+        'SH': True,
+        'SHN': True,
+        'IQIN': True,
+        'DD': True,
+        }
     MAX_FSHIFT = {'ZIF': 62.5*M}
     SWEEP_SETTINGS = ['rfe_mode', 'fstart', 'fstop', 'fstep', 'fshift',
         'decimation', 'attenuator', 'ifgain', 'spp', 'ppb',
         'dwell_s', 'dwell_us',
         'trigtype', 'level_fstart', 'level_fstop', 'level_amplitude']
 
+    LEVEL_TRIGGER_RFE_MODES = ['SH', 'SHN', 'ZIF']
+
+    SPECA_DEFAULTS = {
+        'mode': 'ZIF',
+        'center': 2450 * M,
+        'rbw': 122070,
+        'span': 125 * M,
+        'decimation': 1,
+        'fshift': 0,
+        'device_settings': {
+            'attenuator': True,
+            'iq_output_path': 'DIGITIZER',
+            'pll_reference': 'INT',
+            'trigger': {'type': 'NONE',
+                        'fstart': 2440 * M,
+                        'fstop': 2460 * M,
+                        'amplitude': -110},
+            },
+        'device_class': 'thinkrf.WSA',
+        'device_identifier': 'unknown',
+        }
+
+
 class WSA5000_220_v2Properties(WSA5000_220Properties):
     model = 'WSA5000-220 v2'
     # v2 -> hardware revision without SHN mode
@@ -154,6 +256,7 @@ class WSA(object):
             self.properties = WSA5000_220Properties
 
         self.fw_version = device_id.split(',')[-1]
+        self.device_state = {}
 
     def disconnect(self):
         """
@@ -213,7 +316,7 @@ class WSA(object):
             mode = buf.strip()
         else:
 
-            self.scpiset(":INPUT:MODE %s" % mode)
+            self.scpiset(":INPUT:MODE %s" % str(mode))
 
         yield mode
 
@@ -424,43 +527,30 @@ class WSA(object):
         enable the trigger engine.
 
         :param settings: the new trigger settings; None to query
-        :type settings: pyrf.config.TriggerSettings
+        :type settings: dictionary
         :returns: the trigger settings
         """
         if settings is None:
             # find out what kind of trigger is set
             trigstr = yield self.scpiget(":TRIGGER:TYPE?")
-            if trigstr == "NONE":
-                settings = TriggerSettings("NONE")
-
-            elif trigstr == "LEVEL":
-                # build our return object
-                settings = TriggerSettings("LEVEL")
-
+            if trigstr == "LEVEL":
                 # read the settings from the box
                 trigstr = yield self.scpiget(":TRIGGER:LEVEL?")
-                settings.fstart, settings.fstop, settings.amplitude = trigstr.split(",")
-
-                # convert to integers
-                settings.fstart = int(settings.fstart)
-                settings.fstop = int(settings.fstop)
-                settings.amplitude = float(settings.amplitude)
-
+                settings = {"type": trigstr,
+                            "fstart": int(trigstr.split(",")[0]),
+                            "fstop": int(trigstr.split(",")[1]),
+                            "amplitude": int(trigstr.split(",")[2])}
             else:
-                raise TriggerSettingsError("unsupported trigger type set: %s" % trigstr)
-
+                settings = {"type": trigstr}
         else:
-            if settings.trigtype == "NONE":
-                self.scpiset(":TRIGGER:TYPE NONE")
-
-            elif settings.trigtype == "LEVEL":
-                self.scpiset(":TRIGGER:LEVEL %d, %d, %d" % (settings.fstart, settings.fstop, settings.amplitude))
-                self.scpiset(":TRIGGER:TYPE LEVEL")
+            self.scpiset(":TRIGGER:TYPE %s" % settings["type"])
 
+            if settings["type"] == "LEVEL":
+                self.scpiset(":TRIGGER:LEVEL %d, %d, %d" % (settings['fstart'], 
+                                                            settings['fstop'], 
+                                                            settings['amplitude']))
         yield settings
 
-
-
     def capture(self, spp, ppb):
         """
         This command will start the single block capture and the return of
@@ -772,10 +862,16 @@ class WSA(object):
             'rfe_mode': self.rfe_mode,
             'iq_output_path': self.iq_output_path,
             'pll_reference': self.pll_reference,
+            'trigger': self.trigger,
             }
-
         for k, v in settings.iteritems():
-            device_setting[k](v)
+            #FIXME: Find more elegant way to do this
+            if not k in self.device_state:
+                self.device_state[k] = v
+                device_setting[k](v)
+            if not self.device_state[k] == v:
+                self.device_state[k] = v
+                device_setting[k](v)
 
 
 def parse_discovery_response(response):
